@@ -11,7 +11,15 @@ geom_hline <- function(mapping = NULL, data = NULL,
 
   # Act like an annotation
   if (!missing(yintercept)) {
-    data <- data.frame(yintercept = yintercept)
+    # Warn if supplied mapping and/or data is going to be overwritten
+    if (!is.null(mapping)) {
+      cli::cli_warn("{.fn geom_hline}: Ignoring {.arg mapping} because {.arg yintercept} was provided.")
+    }
+    if (!is.null(data)) {
+      cli::cli_warn("{.fn geom_hline}: Ignoring {.arg data} because {.arg yintercept} was provided.")
+    }
+
+    data <- data_frame0(yintercept = yintercept)
     mapping <- aes(yintercept = yintercept)
     show.legend <- FALSE
   }
@@ -24,7 +32,7 @@ geom_hline <- function(mapping = NULL, data = NULL,
     position = PositionIdentity,
     show.legend = show.legend,
     inherit.aes = FALSE,
-    params = list(
+    params = list2(
       na.rm = na.rm,
       ...
     )
@@ -36,19 +44,21 @@ geom_hline <- function(mapping = NULL, data = NULL,
 #' @usage NULL
 #' @export
 GeomHline <- ggproto("GeomHline", Geom,
-  draw_panel = function(data, panel_params, coord) {
-    ranges <- coord$range(panel_params)
+  draw_panel = function(data, panel_params, coord, lineend = "butt") {
+    ranges <- coord$backtransform_range(panel_params)
 
     data$x    <- ranges$x[1]
     data$xend <- ranges$x[2]
     data$y    <- data$yintercept
     data$yend <- data$yintercept
 
-    GeomSegment$draw_panel(unique(data), panel_params, coord)
+    GeomSegment$draw_panel(unique0(data), panel_params, coord, lineend = lineend)
   },
 
-  default_aes = aes(colour = "black", size = 0.5, linetype = 1, alpha = NA),
+  default_aes = aes(colour = "black", linewidth = 0.5, linetype = 1, alpha = NA),
   required_aes = "yintercept",
 
-  draw_key = draw_key_path
+  draw_key = draw_key_path,
+
+  rename_size = TRUE
 )
