@@ -7,7 +7,7 @@
 #' `y`, `width`, `height`). `geom_raster()` is a high
 #' performance special case for when all the tiles are the same size.
 #'
-#' @eval rd_aesthetics("geom", "tile")
+#' @eval rd_aesthetics("geom", "tile", "Note that `geom_raster()` ignores `colour`.")
 #' @inheritParams layer
 #' @inheritParams geom_point
 #' @inheritParams geom_segment
@@ -40,6 +40,7 @@
 #' \donttest{
 #' # Justification controls where the cells are anchored
 #' df <- expand.grid(x = 0:5, y = 0:5)
+#' set.seed(1)
 #' df$z <- runif(nrow(df))
 #' # default is compatible with geom_tile()
 #' ggplot(df, aes(x, y, fill = z)) +
@@ -82,7 +83,7 @@ geom_tile <- function(mapping = NULL, data = NULL,
     position = position,
     show.legend = show.legend,
     inherit.aes = inherit.aes,
-    params = list(
+    params = list2(
       linejoin = linejoin,
       na.rm = na.rm,
       ...
@@ -108,10 +109,15 @@ GeomTile <- ggproto("GeomTile", GeomRect,
     )
   },
 
-  default_aes = aes(fill = "grey20", colour = NA, size = 0.1, linetype = 1,
+  default_aes = aes(fill = "grey20", colour = NA, linewidth = 0.1, linetype = 1,
     alpha = NA, width = NA, height = NA),
 
   required_aes = c("x", "y"),
 
+  # These aes columns are created by setup_data(). They need to be listed here so
+  # that GeomRect$handle_na() properly removes any bars that fall outside the defined
+  # limits, not just those for which x and y are outside the limits
+  non_missing_aes = c("xmin", "xmax", "ymin", "ymax"),
+
   draw_key = draw_key_polygon
 )
